package com.muyu.cloud.etl.service.impl;

import com.muyu.Hikari.HikariPool;
import com.muyu.cloud.etl.mysql.MySqlDataSource;
import com.muyu.cloud.etl.mysql.MySqlQuery;
import com.muyu.cloud.etl.service.DataValueService;
import com.muyu.cloud.etl.service.SourceService;
import com.muyu.cloud.etl.service.TableInfoService;
import com.muyu.data.base.BaseQueryHandler;
import com.muyu.domain.DataValue;
import com.muyu.domain.Source;
import com.muyu.domain.TableInfo;
import com.muyu.domain.enums.DataType;
import com.zaxxer.hikari.HikariDataSource;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.sql.*;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.ConcurrentHashMap;

@Service
@Log4j2
public class DataValueServiceImpl implements DataValueService {
    @Autowired
    private SourceService sourceService;
    @Autowired
    private TableInfoService tableInfoService;

    @Override
    public List<List<DataValue>> findTableValue(Long basicId, String sql) {
        ConcurrentHashMap<Integer, DataValue> map = new ConcurrentHashMap<>();
        Source source = sourceService.getInfo(basicId);

//        HikariConfig hikariConfig = new HikariConfig();
//        hikariConfig.setPoolName("HikariCP 连接池");
//        hikariConfig.setDriverClassName(source.getDriverName());
//        hikariConfig.setJdbcUrl(source.getUrl());
//        hikariConfig.setUsername(source.getUsername());
//        hikariConfig.setPassword(source.getPassword());
//        hikariConfig.setMinimumIdle(2);
//        hikariConfig.setMaximumPoolSize(10);
//
//        HikariDataSource hikariDataSource = new HikariDataSource(hikariConfig);

        HikariDataSource hikariDataSource = HikariPool.getHikariDataSource(source);

        List<List<DataValue>> list = new ArrayList<>();
        Connection conn=null;

        try {
            ArrayList<DataValue> dataValues = new ArrayList<>();
            conn = hikariDataSource.getConnection();
            try {
                PreparedStatement preparedStatement = conn.prepareStatement(sql);
                ResultSet resultSet = preparedStatement.executeQuery();
                ResultSetMetaData metaData = resultSet.getMetaData();
                int columnCount = metaData.getColumnCount();
                while (resultSet.next()){
                    for (int i = 1; i <= columnCount; i++) {
                        if (resultSet.isFirst()){
                            String columnTypeName = metaData.getColumnTypeName(i);

                            DatabaseMetaData metaData1 = conn.getMetaData();

                            ResultSet columns = metaData1.getColumns(null, null, metaData.getTableName(i), metaData.getColumnName(i));

                            String remarks =null;

                            while (columns.next()){
                                remarks = columns.getString("REMARKS");
                                log.info("字段备注:"+remarks);
                            }
                            DataValue build = DataValue.builder()
                                    .key(metaData.getColumnName(i))
                                    .label(remarks)
                                    .value(resultSet.getObject(i, DataType.convertType(columnTypeName)))
                                    .type(DataType.findBySqlType(columnTypeName))
                                    .build();
                            dataValues.add(build);
                            map.put(i,build);
                        }else {
                            DataValue build = DataValue.builder()
                                    .key(metaData.getColumnName(i))
                                    .label(map.get(i).getLabel())
                                    .value(resultSet.getObject(i, map.get(i).getType().getTargetType()))
                                    .type(map.get(i).getType())
                                    .build();
                            dataValues.add(build);
                        }

                    }
                    list.add(dataValues);

                }


            } catch (SQLException e) {
                throw new RuntimeException(e);
            }
        } catch (SQLException e) {
            throw new RuntimeException(e);
        }


        return list;
    }

    @Override
    public List<DataValue> findTableValueList(Long basicId, String tableName) {
        Source source = sourceService.getInfo(basicId);
        String host = source.getHost();
        String port = source.getPort();
        String databaseName = source.getDatabaseName();
        String databaseType = source.getDatabaseType();
        String url = "jdbc:" + databaseType + "://" + host + ":" + port + "/" + databaseName + "?" + source.getConnectionParams();
        String user = source.getUsername();
        String password = source.getPassword();
        Connection conn=null;
        ArrayList<DataValue> dataValues = new ArrayList<>();
        try {

            conn = DriverManager.getConnection(url, user, password);
            try {
                PreparedStatement preparedStatement = conn.prepareStatement("SELECT  * FROM " + tableName);
                ResultSet resultSet = preparedStatement.executeQuery();
                ResultSetMetaData metaData = resultSet.getMetaData();
                int columnCount = metaData.getColumnCount();
                while (resultSet.next()){
                    for (int i = 1; i <= columnCount; i++) {

                        String columnTypeName = metaData.getColumnTypeName(i);

                        DatabaseMetaData metaData1 = conn.getMetaData();

                        ResultSet columns = metaData1.getColumns(null, null, metaData.getTableName(i), metaData.getColumnName(i));

                        String remarks =null;

                        while (columns.next()){
                            remarks = columns.getString("REMARKS");
                            log.info("字段备注:"+remarks);
                        }
                        DataValue build = DataValue.builder()
                                .key(metaData.getColumnName(i))
                                .label(remarks)
                                .value(resultSet.getObject(i, DataType.convertType(columnTypeName)))
                                .type(DataType.findBySqlType(columnTypeName))
                                .build();
                        dataValues.add(build);
                    }

                }
            } catch (SQLException e) {
                throw new RuntimeException(e);
            }
        } catch (SQLException e) {
            throw new RuntimeException(e);
        }


        return dataValues;
    }


    //供连接数据库调用公共方法
    private Connection getConnection(Source source) {
        Connection conn = null;
        String host = source.getHost();
        String port = source.getPort();
        String databaseName = source.getDatabaseName();
        String databaseType = source.getDatabaseType();
        String url = "jdbc:" + databaseType + "://" + host + ":" + port + "/" + databaseName + "?" + source.getConnectionParams();
        String user = source.getUsername();
        String password = source.getPassword();
        try {
            return DriverManager.getConnection(url,user,password);
        } catch (SQLException e) {
            throw new RuntimeException(e);
        }

    }

    @Override
    public Integer addTableDataValue(Long basicId, Long tableId, List<List<DataValue>> dataValueList) {
        // 获取数据源信息
        Source source = sourceService.getInfo(basicId);
        TableInfo tableInfo = tableInfoService.getById(tableId);
        HikariDataSource hikariDataSource = HikariPool.getHikariDataSource(source);
        Connection conn = null;
        try {
            conn = hikariDataSource.getConnection();
            // 获取表名
            String tableName = tableInfo.getTableName();

            // 遍历所有的数据值列表
            PreparedStatement pstmt = null; // 声明预编译语句对象
            for (List<DataValue> dataValues : dataValueList) {
                // 初始化列名和占位符字符串
                StringBuilder columns = new StringBuilder("(");
                StringBuilder placeholders = new StringBuilder("VALUES (");
                ArrayList<String> columnNames = new ArrayList<>();

                // 遍历单个数据集中的所有键值对
                for (DataValue dataValue : dataValues) {
                    String key = dataValue.getKey();
                    Object value = dataValue.getValue();

                    // 构建列名部分
                    columns.append(key).append(",");
                    // 构建占位符部分
                    placeholders.append("?,");
                    // 收集列名用于后续处理
                    columnNames.add(key);
                }

                // 移除最后一个逗号
                columns.deleteCharAt(columns.length() - 1);
                placeholders.deleteCharAt(placeholders.length() - 1);
                // 关闭括号
                columns.append(")");
                placeholders.append(")");

                // 构造完整的SQL语句
                String sql = "INSERT INTO " + tableName + " " + columns.toString() + " " + placeholders.toString();
                log.info("添加sql语句{},",sql);

                // 如果有旧的预编译语句，先关闭它
                if (pstmt != null) pstmt.close();
                // 准备新的预编译语句
                pstmt = conn.prepareStatement(sql);

                // 设置参数值
                int index = 1;
                for (DataValue dataValue : dataValues) {
                    Object value = dataValue.getValue();
                    if (value instanceof String) {
                        pstmt.setString(index, (String) value);
                    } else if (value instanceof Integer) {
                        pstmt.setInt(index, (Integer) value);
                    } else if (value instanceof Double) {
                        pstmt.setDouble(index, (Double) value);
                    } else if (value instanceof Date) {
                        pstmt.setDate(index, new java.sql.Date(((Date) value).getTime()));
                    } else {
                        pstmt.setObject(index, value);
                    }
                    index++;
                }

                // 将语句添加到批处理中
                pstmt.addBatch();
            }

            // 执行所有批处理语句
            if (pstmt != null) {
                pstmt.executeBatch();
            }

        } catch (SQLException e) {
            // 抛出运行时异常，将原始的 SQLException 作为原因
            throw new RuntimeException(e);
        } finally {
            // 在finally块中确保连接关闭
            if (conn != null) {
                try {
                    conn.close();
                } catch (SQLException e) {
                    // 处理或记录关闭连接时发生的异常
                }
            }
        }

// 返回一个指示操作成功的值
        return 1;
    }

    @Override
    public Integer findCount(Long basicId, String sql) {
        Source source = sourceService.getInfo(basicId);
        String host = source.getHost();
        String port = source.getPort();
        String databaseName = source.getDatabaseName();
        String databaseType = source.getDatabaseType();
        String url = "jdbc:" + databaseType + "://" + host + ":" + port + "/" + databaseName + "?" + source.getConnectionParams();
        String user = source.getUsername();
        String password = source.getPassword();
        Connection conn=null;

        Integer string=0;
        try {
            conn = DriverManager.getConnection(url, user, password);
            PreparedStatement prepareStatement = conn.prepareStatement(sql);
            ResultSet resultSet = prepareStatement.executeQuery();
            while (resultSet.next()){
                string = resultSet.getInt(1);
            }


        } catch (SQLException e) {
            throw new RuntimeException(e);
        }

        return string;
    }

    @Override
    public DataValue[][] findTableValueArray(Long basicId, String sql, Long one, Integer two) {

        MySqlQuery mySqlQuery = new MySqlQuery();
        mySqlQuery.setSql(sql);
        mySqlQuery.setDataSourceId(basicId);
        mySqlQuery.setOne(one);
        mySqlQuery.setTwo(two);
        BaseQueryHandler.set(mySqlQuery);
        MySqlDataSource mySqlDataSource = new MySqlDataSource();
        DataValue[][] rows = mySqlDataSource.getRows();

        return rows;
    }
}
